<?php
# rrcountdescendants.php  - Displays the count of descendants of a particular person, broken out by generation.
# Derived from descendtext.php by Robin Richmond and packaged in the Count Descendants mod.
# All of the code having to do with names, dates, and places has been removed.
# Note that descendtext.php does not display spouses in the last generation, if the generations are limited by
# a generation count. This program modifies the logic to scan one more generation than the limit calls for,
# so it can count those spouses.
# This program also defaults to print all generations, rather than the default 4 generations 
# (or whatever the $pedigree['initdescgens'] parameters value is) used by the descendant charts.
# The links from the other Descendant Charts don't pass the generation count the other programs were
# using, though this program still allows you to change the generation count.
# THIS PROGRAM COUNTS DESCENDANTS OF MARRIED COUSINS ONLY ONCE
#
####################################
# v8 15 Apr 2018
# - Added additional results:
#   o Two surname clouds showing the number of instances of surnames - one for descendants, and one for spouses.
#   o A table of instances of surnames by generation
#   o Renamed the counting program to rrcountdescendants.php to match the name of the mod and flag it with "rr"
# v7 21 Nov 2016
# - Followed Roger Mitchell's suggestion to add borders to the results table, and
#   added a mod Parameter that can suppress the border & heading styling.
# v6 24 May 2015 
# - Added a new footer that displays the program name and version.
# v2 11 Apr 2015 
# - Bernard Wortelboer caught some untranslated strings.

$tngModNames['Count Descendants']['rrcountdescendants.php'] = '8'; #Register this mod

# The mod parameter determines whether to style the headings and borders of the results tables 
$tablestyling = 1;
if ($tablestyling) {
	$theadclass = " class='fieldname'";
	$thclass = " class='fieldnameback'";
	$tdclass = " class='lightback'";
	$thcellclass = " class='fieldname fieldnameback'";
} else $theadclass = $thclass = $tdclass = '';

$textpart = "pedigree";
include("tng_begin.php");

if(!$personID) die("no args");
include($subroot . "pedconfig.php");

$getperson_url = getURL( "getperson", 1 );
$descend_url = getURL( "descend", 1 );
$descendtext_url = getURL( "descendtext", 1 );
$desctracker_url = getURL( "desctracker", 1 );
$register_url = getURL( "register", 1 );
$pdfform_url = getURL( "rpt_pdfform", 1 );

$countem = array(); //The descendant & spouse counts
$descendants = array(); //All descendants, in order to avoid duplication
$spouses = array(); //All spouses, in order to avoid duplication
$duplicates = array(); //Two dimensions; "count", and "list" in one, and a count and a list in the other.

$level = 1;
$key = $personID;

$result = getPersonFullPlusDates($tree, $personID);
if( $result ) {
	$row = tng_fetch_assoc( $result );
	$righttree = checktree($tree);
	$rightbranch = $righttree ? checkbranch($row['branch']) : false;
	$rights = determineLivingPrivateRights($row, $righttree, $rightbranch);
	$row['allow_living'] = $rights['living'];
	$row['allow_private'] = $rights['private'];
	$namestr = getName( $row );
	$logname = $tngconfig['nnpriv'] && $row['private'] ? $admtext['text_private'] : ($nonames && $row['living'] ? $text['living'] : $namestr);
}

$treeResult = getTreeSimple($tree);
$treerow = tng_fetch_assoc($treeResult);

$disallowgedcreate = $treerow['disallowgedcreate'];
$allowpdf = !$treerow['disallowpdf'] || ($allow_pdf && $rightbranch);
tng_free_result($treeResult);

writelog( "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree\">{$text['rrcd-descendfor']} $logname ($personID)</a>" );
preparebookmark( "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree\">{$text['rrcd-descendfor']} $namestr ($personID)</a>" );

$flags['tabs'] = $tngconfig['tabs'];
$flags['scripting'] = "<script type=\"text/javascript\">var tnglitbox;</script>\n";
tng_header( $text['rrcd-descendcountfor'] . " $namestr", $flags );
?>
<script type="text/javascript">
// jQuery function to show/hide a list, and to switch between buttons that say "show duplicates" or "hide duplicates"
function showlist(thisid) {
	if (jQuery("#"+thisid+"but").val()=="<?php echo $text['rrcd-showthem']; ?>") {
		jQuery("#"+thisid).show();
		jQuery("#"+thisid+"but").val("<?php echo $text['rrcd-hidethem']; ?>");
	} else {
		jQuery("#"+thisid).hide();
		jQuery("#"+thisid+"but").val("<?php echo $text['rrcd-showthem']; ?>");
	}
} //end function showlist

//////////////////////////////////////////////////////////
//Function to pop up a list of descendants or spouses with a given surname.
//Descendant lists actually start with the corresponding spouses.
//The lists are stored in hidden divs whose ID's include surname numbers.
//The count of people in the list is also stored in a hidden div. We need that count to
//define the length of the popup LITBox.
//Arguments:
// - personType = 'd' for descendants or 's' for spouses.
//     For descendants, both the spouses and descendants will be displayed.
// - heading = the text that will go in the LITBox header
// - surname 
// - surnameNumber - corresponds to the surname, and is used to identify the hidden data.
//   There are two blocks of data: the list of names and the count of names. 
function showNameList(personType, heading, surname, surnameNumber) {
	var width = 500; //personType=='Descendants' ? 325 : 
	var listsize = +$('#rrsc' + surnameNumber).html();
	var xyz1,xyz2; xyz1=listsize;
	var content = '';
	if (listsize>0) {
		content += "<?php echo $text['rrcd-countspouses']; ?>" + $('#rrs' + surnameNumber).prop('outerHTML');
	}
	if (personType == 'd') { //d => display spouses and descendants
		content += "<b><?php echo $text['rrcd-countdescendants']; ?></b>" + $('#rrd' + surnameNumber).prop('outerHTML');
		//Add 2 to the list length for the subheadings
		listsize = +listsize + 2 + +$('#rrdc' + surnameNumber).html();
		xyz2=listsize;
	}
	//The LITBox height is 25px for the heading, plus 15px for each line, with a limit of 40 lines.
	var height = 25 + 12 * (listsize>40 ? 40 : listsize)+1;
	var title = heading +  surname;
	new LITBox(content,{type:'alert', width:width, height:height, overlay:false, title:title});
	return false;
} //End function showPopupHint
</script>
<style>
h4 {margin-bottom:0;}
/* Formatting for the table of descendant and spouse counts (irrespective of name) by generation. */
#counttable td {text-align:right; 
	padding-right:0.5em;
#counttable th {padding: 0 0.5em;}
	border-width:{$borderwidth}px;}
	
/* The hyperlinked names in the surname clouds will not have underlines until the user mouses over them */
#descnames {border:thin solid grey; margin-bottom:1em;}
#descnames a {text-decoration:none;}
#descnames a:hover {text-decoration:underline;}
#spousenames {border:thin solid grey; margin-bottom:1em;}
#spousenames a {text-decoration:none;}
#spousenames a:hover {text-decoration:underline;}
#descdup {margin-bottom:1em;}
#spousedup {margin-bottom:1em;}

.LB_content ol {margin-top:0;margin-bottom:0;}

/* Formatting for the table of surname counts by generation */ 
table.surnametable thead th {text-align:right !important; padding:1px 2px;}
table.surnametable tbody th {padding:1px 2px;}
table.surnametable td {text-align:right;
	padding:0 2px;
	width:2em;
}
</style>

<?php
	$photostr = showSmallPhoto( $personID, $namestr, $rights['both'], 0, false, $row['sex'] );
	echo tng_DrawHeading( $photostr, $namestr, getYears( $row ) );

	if( !$generations ) $generations =  0;
	else $generations = intval( $generations );

	$innermenu = $text['generations'] . ": &nbsp;";
	$innermenu .= "<select name=\"generations\" class=\"verysmall\" onchange=\"window.location.href='rrcountdescendants.php?personID=$personID&amp;tree=$tree&amp;display=$display&amp;generations=' + this.options[this.selectedIndex].value\">\n";
    $innermenu .= "<option value=\"99\"";
    if ($generation == 0 ) $innermenu .= " selected=\"selected\"";
    $innermenu .= ">{$text['all']}</option>\n";
    for( $i = 4; $i <= 25; $i++ ) {
        $innermenu .= "<option value=\"$i\"";
        if( $i == $generations ) $innermenu .= " selected=\"selected\"";
        $innermenu .= ">$i</option>\n";
    }
	$innermenu .= "</select>&nbsp;&nbsp;&nbsp;\n";
	$innermenu .= "<a href=\"$descend_url" . "personID=$personID&amp;tree=$tree&amp;display=standard&amp;generations=$generations\" class=\"lightlink\">{$text['pedstandard']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$descend_url" . "personID=$personID&amp;tree=$tree&amp;display=compact&amp;generations=$generations\" class=\"lightlink\">{$text['pedcompact']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree&amp;generations=$generations\" class=\"lightlink\">{$text['pedtextonly']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	#Include the rrtextplus_descend hyperlink only if the rrtextplus_descend program is present.
	if (file_exists('rrtextplus_descend.php')) 
		$innermenu .= "<a href=\"rrtextplus_descend.php?personID=$personID&amp;tree=$tree&amp;generations=$generations\" class=\"lightlink\">{$text['rrcd-textplus']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$register_url" . "personID=$personID&amp;tree=$tree&amp;generations=$generations\" class=\"lightlink\">{$text['regformat']}</a>\n";
	if($generations <= 12 && $allowpdf)
		$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href=\"#\" class=\"lightlink\" onclick=\"tnglitbox = new LITBox('$pdfform_url" . "pdftype=desc&amp;personID=$personID&amp;tree=$tree&amp;generations=$generations',{width:350,height:350});return false;\">PDF</a>\n";
	$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href=\"rrcountdescendants.php?personID=$personID&amp;tree=$tree&amp;generations=$generations\" class=\"lightlink3\">{$text['rrcd-descendcount']}</a>\n";

	echo getFORM( "descend", "get", "form1", "form1" );
	echo tng_menu( "I", "descend", $personID, $innermenu );
	echo "</form>\n";
?>
<div class="normal" id="descendcount">
<?php

########## Output the name of the starting person a subheading to the report. No need for a link here.
$vitalinfo = getVitalDates( $row );
echo "<p>$namestr&nbsp; $vitalinfo</p>\n";
$countem[1]['d']++;
$surname = markUnknown(strtoupper($row['lastname']));
$surnames['d'][$surname] = 1;
#Save the linked surname in the $names list
$namestr = "<a href='getperson.php?tree=$tree&personID=$personID'>$namestr</a>";
$names['d'][$surname][1][$personID] = $namestr;
$descendants[$key] = 1;

if ($generations == 0) $generations = 99;  //Zero means do all
if( $generations > 1 ) {
	$ind = getIndividual( $key, $row['sex'], $level + 1, $namestr );
}

$dcount=0;
$scount=0;
echo "<table id='counttable' cellspacing='1' cellpadding='3'>
<thead $theadclass>
	<tr><th $thclass></th><th $thclass></th>
		<th $thclass>{$text['rrcd-countdescendants']}</th>
		<th $thclass>{$text['rrcd-countspouses']}</th>
		<th $thclass>{$text['rrcd-counttotal']}</th>
</tr>
</thead>
<tbody>
"; #End of multi-line echo statement

//Don't try to print more generations than exist.
if ( $generations >count($countem) ) $generations=count($countem);
//Step through the generations, printing the counts.
for ($i=1; $i<=$generations; $i++) {
	echo "<tr><td $tdclass>$i.&nbsp;</td><td $tdclass>";
	switch ($i) {
		case 1: echo $text['rrcd-self']; break;
		case 2: echo $text['children']; break;
		case 3: echo $text['rrcd-grandchildren']; break;
		case 4: echo $text['rrcd-greatgrand']; break;
		case 5:
			if ($text['rrcd-greatgreatgrand'] ) echo $text['rrcd-greatgreatgrand'];
			else echo "{$text['rrcd-ggg']}*" . ($i-3) ." {$text['rrcd-lcgrandchildren']}";
			break;
		default: 
			$repeat = ($text['greatgreatgrand'] ) ? ($i-4) : ($i-3);
			echo "{$text['rrcd-ggg']}*" . $repeat ." {$text['rrcd-lcgrandchildren']}";
	}
	//Now show the count for this generation
	echo "</td><td $tdclass>{$countem[$i]['d']}</td>
		<td $tdclass>". +$countem[$i]['s'] . "</td>
		<td $tdclass>" . ($countem[$i]['d']+$countem[$i]['s']) ."</td></tr>\n";
	$dcount += $countem[$i]['d'];  //Accumulate descendants
	$scount += $countem[$i]['s'];  //Accumulate spouses
}
//Now the grand totals
echo "<tr><td $tdclass></td>
		<td $tdclass><strong>{$text['rrcd-counttotal']}</strong></td>
		<td $tdclass><strong>$dcount</strong></td>
		<td $tdclass><strong>$scount</strong></td>
		<td $tdclass><strong>". ($dcount+$scount) . "</strong></td></tr>
</table>\n";
//First conditional message "There are more generations..."
if ( $countem[$generations+1]['d'] ) echo "<div><strong>{$text['rrcd-moredescendants']}</strong></div>\n";

#If we're displaying tables without borders and styling, generate a horizontal line
if (!$tablestyling) echo "<hr>";

//First list - descendants who married cousins
if ( $duplicates['s']['count'] ) {
	echo "<strong>{$text['rrcd-cousinmarr']}|Marriages between cousins</strong>: {$duplicates['s']['count']}";
	echo "&nbsp;<input type='button' id='cousinmarrbut' value='{$text['rrcd-showthem']}' onclick=\"showlist('cousinmarr')\" /><br/>\n";
	echo "<ol id='cousinmarr' style='display:none;margin-top:0;'>";
	echo "{$duplicates['s']['list']}</ol>";
}
//Second list - spouses who married more than one descendant.
if ( $duplicates['ss']['count'] ) {
	echo "<strong>{$text['rrcd-spousemore']}|Spouses who married more than one descendant</strong>: {$duplicates['ss']['count']}";
	echo "&nbsp;<input type='button' id='spousemorebut' value='{$text['rrcd-showthem']}' onclick=\"showlist('spousemore')\" /><br/>\n";
	echo "<ol id='spousemore' style='display:none;margin-top:0;'>";
	echo "{$duplicates['ss']['list']}</ol>";
}

//Third list - descendants who have multiple sets of parents
if ( $duplicates['p']['count'] ) {
	echo "<strong>{$text['rrcd-parentdup']}|Descendants who have two sets of parents</strong>: {$duplicates['p']['count']}";
	echo "&nbsp;<input type='button' id='parentdupbut' value='{$text['rrcd-showthem']}' onclick=\"showlist('parentdup')\" /><br/>\n";
	echo "<ol id='parentdup' style='display:none;margin-top:0;'>";
	echo "{$duplicates['p']['list']}</ol>";
}

#Fourth list - Descendant surname cloud
echo "<strong>{$text['rrcd-descsurn']}|Descendant Surnames</strong>: ",count($surnames['d']);
echo "&nbsp;<input type='button' id='descnamesbut' value='{$text['rrcd-showthem']}' 
	onclick=\"showlist('descnames')\" /><br/>\n";

asort($surnames['d']);
#Sequence the counts in a way that will normalize the relative sizes of the names in the surname cloud.
#That is, we don't want a name with 100 occurrences to be 100 times larger that a name with 1 occurrence
$max = $lastcount = 1;
$sizesort = array(1=>1);
foreach ($surnames['d'] as $surname=>$count) {
	if (substr($surname, 0, 1) == '[')
		#This is a name that means "unknown" - don't increase the font size above the 
		#size just before this name.
		$sizesort[$count] = $sizesort[$lastcount];
	elseif ($count>$lastcount+1) {
		$max = $sizesort[$count] = $sizesort[$lastcount] + 3 + round(sqrt($count-$lastcount)+.5);
		$lastcount = $count;
	} elseif ($count == $lastcount+1) {
		$max = $sizesort[$count]= $sizesort[$lastcount] + 3;
		$lastcount = $count;
	}
}
#The surname cloud is ordered by name
ksort($surnames['d']);
# $surnameCount is incremented for each name, and its values are stores in the $surnameNumbers array
# which is indexed by surname.
$surnameNumbers = array();
$surnameCount = 0;
# $html1 builds the surname cloud that can be displayed by a button.
# $html2 builds the lists of people for each surname.  Those lists are stored in always-hidden 
# HTML blocks, and displayed by the showNameList JavaScript function.
$html1 = $html2 = '';
$ratio = 130./$max; #Font size ratio for the surname cloud
foreach ($surnames['d'] as $surname=>$peopleCount) {
	$surnameNumbers[$surname] = ++$surnameCount;
	$fontsize = round( $ratio * $sizesort[$peopleCount]) +80;
	$html1 .= "<a style='font-size:$fontsize%' href='#' 
		onclick=\"showNameList('d', '{$text['rrcd-peoplewith']}|People with surname ', '$surname', $surnameCount);\" 
		title='$peopleCount descendants'>$surname</a> ";
	
	$html2 .= "<ol id='rrd$surnameCount'>";
	foreach ($names['d'][$surname] as $generation => $people) {
		foreach ($people as $personID => $namestr)
			$html2 .= "<li>Gen $generation: $namestr</li>\n";
	}
	$html2 .= "</ol>\n";
	#Add a hidden div with the count of descendants with this surname
	$html2 .= "<div id='rrdc$surnameCount'>$peopleCount</div>";
	if (!array_key_exists($surname, $surnames['s'])) {
		#All descendant surnames should have a corresponding spouse surname (except in generation 1).
		#This one doesn't, so create a spouse count of zero.
		$html .= "<div id='rrds$surnameCount'>0</div>\n";
	}
}
echo "<div id='descnames' style='display:none;'>$html1</div>\n";
echo "<div id='desclists' style='display:none;'>$html2</div>\n";

##Fifth list - spouse surname cloud
echo "<strong>{$text['rrcd-spousesurn']}|Spouse Surnames</strong>: ",count($surnames['s']);
echo "&nbsp;<input type='button' id='spousenamesbut' value='{$text['rrcd-showthem']}' onclick=\"showlist('spousenames')\" /><br/>\n";

asort($surnames['s']);
#Sequence the counts in a way that will normalize the relative sizes of the names in the surname cloud.
$max = $lastcount = 1;
$sizesort = array(1=>1);
foreach ($surnames['s'] as $surname=>$count) {
	if (substr($surname, 0, 1) == '[')
		#This is a name that means "unknown" - don't increase the font size above the 
		#size just before this name.
		$sizesort[$count] = $sizesort[$lastcount];
	elseif ($count>$lastcount+1) {
		$max = $sizesort[$count] = $sizesort[$lastcount] + 3+ round(sqrt($count-$lastcount)+.5);
		$lastcount = $count;
	} elseif ($count == $lastcount+1) {
		$max = $sizesort[$count]= $sizesort[$lastcount] + 3;
		$lastcount = $count;
	}
}
ksort($surnames['s']);
# For spouses, $html1 and $html1 have the same meaning as for descendants,
# but we just continue incrementing $surnameCount for each NEW surname.
$html1 = $html2 = '';
# Because we sometimes get only a few occurrences of each spouse surname, we have to fiddle
# with the font size ratio.
if ($max==1) {$ratio = 0; $base=100;}
elseif ($max==2) {$ratio = 10; $base=90;}
elseif ($max==3) {$ratio = 15; $base=85;}
else {$ratio = 60./$max; $base=80;}
foreach ($surnames['s'] as $surname=>$peopleCount) {
	if (isset($surnameNumbers[$surname])) $thisSurnameCount = $surnameNumbers[$surname];
	else $thisSurnameCount = ++$surnameCount;
	$fontsize = round( $ratio * $sizesort[$peopleCount]) + $base;
	$html1 .= "<a style='font-size:$fontsize%' href='#' 
		onclick=\"showNameList('s', '{$text['rrcd-spouseswith']}|Spouses with surname ', '$surname', $thisSurnameCount);\" 
		title='$peopleCount spouses'>$surname</a> ";
	
	$html2 .= "<div id='rrs$thisSurnameCount'><ol>";
	foreach ($names['s'][$surname] as $generation => $people) {
		foreach ($people as $personID => $namestr)
			$html2 .= "<li>Gen $generation: $namestr</li>\n";
	}
	$html2 .= "</ol></div>\n";
	#Add a hidden div with the count of spouses with this surname
	$html2 .= "<div id='rrsc$thisSurnameCount' style='display:none;'>$peopleCount</div>";
}
echo "<div id='spousenames' style='display:none;'>$html1</div>\n";
echo "<div id='spouselists' style='display:none;'>$html2</div>\n";

## Surname/generation table
echo "<strong>{$text['rrcd-descsurncount']}|Descendant Surname Counts by Generation</strong>: ";
echo "&nbsp;<input type='button' id='surnametablebut' value='{$text['rrcd-showthem']}' 
	onclick=\"showlist('surnametable')\" /><br/>\n";

echo "<table id='surnametable' class='surnametable' style='display:none;'>
<thead $theadclass>
<tr><th $thclass>{$text['generations']}:</th>";
for ($ii=1; $ii<=$generations; $ii++)
	echo "<th $thclass>$ii</th>";
echo "</tr></thead>
<tbody>\n";

$html = '';
foreach ($surnames['d'] as $surname=>$count) {
	$html .= "<tr><th $thcellclass>$surname</th>";
	$row = array_fill(1, $generations, '&nbsp;');
	foreach ($names['d'][$surname] as $generation => $people)
		$row[$generation] = count($people);
	for ($ii=1; $ii<=$generations; $ii++)
		$html .= "<td $tdclass>{$row[$ii]}</td>";
	$html .= "</tr>\n";
}
echo "$html</tbody></table>\n";

?>
</div> <!-- end descendcount -->

<script type="text/javascript" src="<?php echo $cms['tngpath']; ?>js/rpt_utils.js"></script>

<?php
tng_footer( "" );

//****************************************************************
//        End of mainline program; Functions follow
//****************************************************************
########################################################################
//Get a hyperlink to the person profile, considering private & living flags and branch access.
//Modeled after getperson.php
function getPersonName ($personrecord) {
	if ( !$personrecord['personID'] ) return "";
	// Ask for the necessary fields from the Person table.
	$personrecord['allow_living'] = 1;
	$rights = determineLivingPrivateRights($personrecord);
	$personrecord['allow_living'] = $rights['living'];
	$personrecord['allow_private'] = $rights['private'];
	return getName( $personrecord );
}

#######################################################################
function getIndividual( $key, $sex, $level, $namestr ) {
	global $tree, $generations, $text, $admtext, $cms, $getperson_url, $righttree;
	global $desctracker_url, $divctr, $display, $excolimg, $descendtext_url, $imgtitle;
	global $countem, $surnames, $names, $descendants, $spouses, $duplicates;
	global $people_table;
	if( $sex == "M" ) {
		$self = "husband";
		$spouse = "wife";
		$spouseorder = "husborder";
	}
	else if( $sex == "F" ){
		$self = "wife";
		$spouse = "husband";
		$spouseorder = "wifeorder";
	}
	else {
		$self = $spouse = $spouseorder = "";
	}

	if( $spouse )
		$result = getSpouseFamilyMinimal($tree, $self, $key, $spouseorder);
	elseif( $key )
		$result = getSpouseFamilyMinimalUnion($tree, $key);
	$marrtot = tng_num_rows($result);
	if( !$marrtot && $key) {
		$result = getSpouseFamilyMinimalUnion($tree, $key);
		$self = $spouse = $spouseorder = "";
	}

	if( $result ) {
		while( $row = tng_fetch_assoc( $result ) ) {
			$spouserow = array();
			$spousestr = "";
			if( !$spouse )
				$spouse = $row['husband'] == $key ? "wife" : "husband";
			$spouseID = $row[$spouse];
			if( $spouseID ) {
				$countem[$level-1]['s']++; //Count a spouse
				#Add the spouse's last name to the surname table.
				$sresult = getPersonSimple($tree, $spouseID);
				$srow = tng_fetch_assoc( $sresult );
				$rightbranch = $righttree ? checkbranch($srow['branch']) : false;
				$srights = determineLivingPrivateRights($srow, $righttree, $rightbranch);
				$srow['allow_living'] = $srights['living'];
				$srow['allow_private'] = $srights['private'];
				$spousestr = getName( $srow );
				$spousestr = "<a href='getperson.php?tree=$tree&personID=$spouseID'>$spousestr</a>";
				$surname = markUnknown(strtoupper($srow['lastname']));
				$surnames['s'][$surname]++;
				$names['s'][$surname][$level-1][$spouseID] = $namestr . ' &amp; ' . $spousestr;

				//If we've seen this spouse before, then this spouse must have married another descendant
				if ( isset($spouses[$spouseID]) ) {
					#Just add the spouse name to the list, since we don't know who the spouse's
					#first descendant-spouse was.
					$duplicates['ss']["count"]++;
					//Get the spouse record so that we can list the spouse
					$spouseresult = getPersonSimple($tree, $spouseID);
					if( $spouseresult ) {
						$spouserow = tng_fetch_assoc( $spouseresult );
						$duplicates['ss']['list'] .= "<li><a href=\"getperson.php?personID=$spouseID&tree=$tree\">" . getPersonName($spouserow) . "</a></li>\n";
					}
					tng_free_result( $spouseresult );
				}
				$spouses[$spouserow['personID']] = 1; // Mark this spouse
				//Have we seen this spouse as a descendant?
				if ( isset($descendants[$spouseID]) ) {
					//Count this as a marriage between cousins
					$duplicates['s']["count"] +=1;
					$familyID = $row['familyID'];
					//Get the husband first
					$spouseID = $row['husband'];
					$spouseresult = getPersonSimple($tree, $spouseID);
					if( $spouseresult ) {
						$spouserow = tng_fetch_assoc( $spouseresult );
						$duplicates['s']['list'] .= "<li><a href=\"getperson.php?personID=$spouseID&tree=$tree\">" . getPersonName($spouserow) . '</a>';
					}
					//Now the wife
					$spouseID = $row['wife'];
					$spouseresult = getPersonSimple($tree, $spouseID);
					if( $spouseresult ) {
						$spouserow = tng_fetch_assoc( $spouseresult );
						$duplicates['s']['list'] .= " &amp; <a href=\"getperson.php?personID=$spouseID&tree=$tree\">" . getPersonName($spouserow) . '</a>';
					}
					$duplicates['s']['list'] .= " - <a href=\"familygroup.php?familyID=$familyID&tree=$tree\">{$text['family']}</a></li>\n";

					tng_free_result( $spouseresult );
					continue; // Skip to next spouse because we've already seen this marriage's descendants
				}
			}

			$result2 = getChildrenSimple($tree, $row['familyID']);
			$numkids = tng_num_rows( $result2 );
			if( $numkids ) {
				while( $crow = tng_fetch_assoc( $result2 ) ) {
					//Don't count or drill down if we've seen this person before
					$personID = $crow['personID'];
					if ( !array_key_exists($personID, $descendants) ) {
						//Not seen before; count and mark this descendant.
						$countem[$level]['d']++;
						#Add the descendant's last name to the surname table.
						$rightbranch = $righttree ? checkbranch($crow['branch']) : false;
						$crights = determineLivingPrivateRights($crow, $righttree, $rightbranch);
						$crow['allow_living'] = $crights['living'];
						$crow['allow_private'] = $crights['private'];
						$namestr = getName( $crow );
						$namestr = "<a href='getperson.php?tree=$tree&personID=$personID'>$namestr</a>";
						$surname = markUnknown(strtoupper($crow['lastname']));
						$surnames['d'][$surname]++;
						$names['d'][$surname][$level][$personID] = $namestr;

						$descendants[$personID] = 1; //Mark this person
						if(!$generations || $level <= $generations ) 
							$ind = getIndividual( $personID, $crow[sex], $level + 1, $namestr);
					} else {
						// The person has already been seen as a descendant. Must have more than one set of parents.
						$duplicates['p']['count']++;
						$duplicates['p']['list'] .= "<li><a href=\"getperson.php?personID=$personID&tree=$tree\">" . getPersonName($crow) . "</a></li>\n";
					}
				}
			}
			tng_free_result( $result2 );
		}
	}
	tng_free_result( $result );
	return "";
} #end function getIndividual

##########################################################################
function getVitalDates( $row ) {
	global $text;
	$vitalinfo = "";

	if( $row['allow_living'] && $row['allow_private'] ) {
		if( $row['birthdate'] ) {
			$vitalinfo = $text['birthabbr'] . " " . displayDate( $row['birthdate'] ) . " ";
		}
		else if( $row['altbirthdate'] ){
			$vitalinfo = $text['chrabbr'] . " " . displayDate( $row['altbirthdate'] ) . " ";
		}
		else {
			$vitalinfo .= " ";
		}
		if( $row['deathdate'] ) {
			$vitalinfo .= $text['deathabbr'] . " " . displayDate( $row['deathdate'] );
		}
		else if( $row['burialdate'] ){
			$vitalinfo .= $text['burialabbr'] . " " . displayDate( $row['burialdate'] );
		}
		else {
			$vitalinfo .= " ";
		}
	}
	return $vitalinfo;
} #end function getVitalDates

#####################################################################
#Mark names that represent unknown surnames by enclosing them in brackets.
#That will put them all at the end of the sorted list of names.
#Names that consist only of underscores, question marks, and/or spaces are considered Unknown
function markUnknown($surname) {
	global $text;
	if ($surname == '') return "[{$text['rrcd-empty']}|empty]";
	###THE NEXT LINE CONTAINS {}|unknown
	if ($surname == strtoupper($text['unknown']) || preg_match( '@^[_\?]*$@', $surname))
		return "[$surname]";
	return $surname;
} #end function markUnknown
?>
